/**
 * taxonomy.C: Lists object properties via their metadata table.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: taxonomy.C,v 1.9 2007/09/22 14:27:28 pippijn Exp $
 */

#include "atheme.h"
#include <ermyth/module.h>
#include <account/myuser.h>

static char const rcsid[] = "$Id: taxonomy.C,v 1.9 2007/09/22 14:27:28 pippijn Exp $";

REGISTER_MODULE ("nickserv/taxonomy", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void ns_cmd_taxonomy (sourceinfo_t *si, int parc, char *parv[]);

command_t const ns_taxonomy = { "TAXONOMY", N_("Displays a user's metadata."), AC_NONE, 1, ns_cmd_taxonomy };

E cmdvec ns_cmdtree;
E helpvec ns_helptree;

bool
_modinit (module *m)
{
  ns_cmdtree << ns_taxonomy;
  help_addentry (ns_helptree, "TAXONOMY", "help/nickserv/taxonomy", NULL);

  return true;
}

void
_moddeinit ()
{
  ns_cmdtree >> ns_taxonomy;
  help_delentry (ns_helptree, "TAXONOMY");
}

static void
ns_cmd_taxonomy (sourceinfo_t *si, int parc, char *parv[])
{
  char *target = parv[0];
  myuser_t *mu;
  bool isoper;
  metadata::map_type::iterator mumdit, mumdit_end;

  if (!target)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "TAXONOMY");
      command_fail (si, fault::needmoreparams, _("Syntax: TAXONOMY <nick>"));
      return;
    }

  if (!(mu = myuser_t::find_ext (target)))
    {
      command_fail (si, fault::badparams, _("\2%s\2 is not registered."), target);
      return;
    }

  isoper = has_priv (si, PRIV_USER_AUSPEX);
  if (isoper)
    logcommand (si, CMDLOG_ADMIN, "TAXONOMY %s (oper)", target);
  else
    logcommand (si, CMDLOG_GET, "TAXONOMY %s", target);

  command_success_nodata (si, _("Taxonomy for \2%s\2:"), target);

  for (mumdit = mu->mdmap.begin (), mumdit_end = mu->mdmap.end (); mumdit != mumdit_end; ++mumdit)
    {
      metadata *md = mumdit->second;
      if (md->is_private == true && !isoper)
        continue;
      
      command_success_nodata (si, "%-32s: %s", md->key, md->value);
    }
  
  command_success_nodata (si, _("End of \2%s\2 taxonomy."), target);
}
